﻿Class DrawFreehandLine : Implements Api.Invoker, ICloneable
    Dim _canvas As Api.Canvas
    Dim _tablet As Tablet
    Dim _revisedStrokePoints As List(Of Api.StrokePoint) = New List(Of Api.StrokePoint)
    Dim _strokePoints As List(Of Api.StrokePoint) = New List(Of Api.StrokePoint)
    Dim _strokePointsIndex As Long = -1
    Dim _image As Image
    Dim _lock = New Threading.ReaderWriterLockSlim
    Dim _commands As List(Of Api.Command)
    Dim _strokeModifier As Api.StrokeModifier

    ReadOnly Property StrokePoints() As List(Of Api.StrokePoint) Implements Api.Invoker.StrokePoints
        Get
            StrokePoints = Me._revisedStrokePoints
        End Get
    End Property

    Sub AddStrokePoint(ByVal strokePoint As Api.StrokePoint) Implements Api.Invoker.AddStrokePoint
        Me._strokePoints.Add(strokePoint)
    End Sub

    Property Canvas() As Api.Canvas Implements Api.Invoker.Canvas
        Get
            Canvas = Me._canvas
        End Get
        Set(ByVal value As Api.Canvas)
            Me._canvas = value
        End Set
    End Property

    Private Sub Invoke() Implements Api.Invoker.Invoke
        With Me._lock
            .EnterWriteLock()
            Me._strokePointsIndex = Me._strokePointsIndex + 1
            Debug.Assert(Me._strokePoints IsNot Nothing, "Invoke()", "Count:" & Me._strokePoints.Count & ", Index:" & Me._strokePointsIndex)
            Me._revisedStrokePoints = Me._strokeModifier.GetRevisedStrokePoints(Me._strokePoints(Me._strokePointsIndex))

            Dim commands As IEnumerator(Of Api.Command) = Me._commands.GetEnumerator()
            Do While commands.MoveNext()
                commands.Current.Invoke(Me)
            Loop
            'Log.add(Api.Invoker)
            .ExitWriteLock()
        End With
    End Sub

    Public Function Clone() As Object Implements System.ICloneable.Clone
        Clone = Me.MemberwiseClone()
        Debug.Assert(Equals(Clone, Me) AndAlso Not ReferenceEquals(Clone, Me))
    End Function

    Sub New(ByVal tablet As Tablet)
        Try
            Dim obj = DirectCast(Me, ICloneable)
        Catch ex As Exception
            Debug.Fail(ex.Message)
        End Try
        Me._tablet = tablet
        Me._commands = Me._tablet.Commands(Me.GetType())
        Me._strokeModifier = Me._tablet.StrokeModifier(Me.GetType())
    End Sub
End Class
